class SearchService

  attr_accessor :search

  def initialize(search, user)
    @search = search
    @user = user
  end

  def fetch
    validate
    return search_learning_object if @search.learning_object?
    return search_collection if @search.collection?
    return search_user if @search.user?
  end

  def autocomplete
    validate

    params_hash = {}
    get_thumbnail = nil
    if @search.learning_object?
      params_hash = { fields: ['name^10', 'description', 'author'] }
      get_thumbnail = Proc.new { |obj| image_tag(learning_object_thumbnail(obj)) }
    elsif @search.collection?
      params_hash = { where: {privacy: "public"},
        fields: ['name^10', 'description', 'owner'] }
      get_thumbnail = Proc.new { |obj| image_tag("/assets/icons/collection") }
    else
      params_hash = { fields: ['name'] }
      get_thumbnail = Proc.new { |obj| image_tag(obj.avatar.url(:thumb), 32) }
    end

    autocomplete_search(Object.const_get(@search.search_class), @search.query, params_hash, get_thumbnail)
  end

  private

  def validate
    raise "Invalid search" unless @search.valid?
  end

  def search_learning_object
    LearningObject.search(@search.query, where: where_hash, order: lo_order_hash(@search.order), page: @search.page, per_page: @search.results_per_page).results
  end
  
  def search_collection
    Collection.search(@search.query, where: {privacy: "public"}, order: col_order_hash(@search.order), page: @search.page, per_page: @search.results_per_page).results
  end

  def search_user
    User.search(@search.query, order: user_order_hash(@search.order), page: @search.page, per_page: @search.results_per_page).results
  end

  def autocomplete_search(search_class, query, params_hash={}, get_thumbnail)
    response = []
    search_params = { limit: 10, misspellings: { below: 5 }, where: {state: validate_object} }
    objs = search_class.search(query, search_params.merge(params_hash))
    objs.each do |obj|
      hash = {}
      hash["name"] = obj.name
      hash["thumbnail"] = get_thumbnail.call(obj)
      hash["url"] = url_for([obj, :only_path => false])
      response <<  hash
    end
    response
  end

  def image_tag(image, width=56, height=32)
    ActionController::Base.helpers.image_tag image, width: width, height: height, class: "autocomplete"
  end

  def where_hash
    hash = {}
    hash[:tags] = @search.tags unless @search.tags.blank?
    hash[:object_type] = @search.types unless @search.types.blank?
    hash[:source] = @search.sources unless @search.sources.blank?
    hash[:state] = validate_object
    hash.blank? ? nil : hash
  end

  def lo_order_hash(order)
    case order
    when 'author'
      { author: {order: :asc, unmapped_type: :string} }
    when 'publicationasc'
      { published_at: {order: :asc, unmapped_type: :timestamp} }
    when 'publicationdesc'
      { published_at: {order: :desc, unmapped_type: :timestamp} }
    when 'title'
      { name: {order: :asc, unmapped_type: :string} }
    else
      { score: {order: :desc, unmapped_type: :integer} }
    end
  end

  def col_order_hash(order)
    case order
    when 'author'
      { owner: {order: :asc, unmapped_type: :string} }
    when 'publicationasc'
      { created_at: {order: :asc, unmapped_type: :timestamp} }
    when 'publicationdesc'
      { created_at: {order: :desc, unmapped_type: :timestamp} }
    when 'title'
      { name: {order: :asc, unmapped_type: :string} }
    else
      { score: {order: :desc, unmapped_type: :integer} }
    end
  end

  def user_order_hash(order)
    if order == 'title'
      { name: {order: :asc, unmapped_type: :string} }
    else
      { score: {order: :desc, unmapped_type: :integer} }
    end
  end

  def validate_object
    return 'published' unless !@user.nil? && @user.is_admin?
    return ['published', 'suspended', 'draft']
  end

end